@@ -2,24 +2,28 @@ import inspect
 import json
 import sys
 import functools
-from _decimal import Decimal
+from decimal import Decimal
 from ipaddress import IPv4Address, IPv6Address
-from typing import Optional, Type, Union, Any, Dict, Tuple, List, TypeVar, get_type_hints, Callable
+from typing import Optional, Type, Union, Any, Dict, Tuple, List, Callable, TypeVar
 import re
-from datetime import datetime
+from datetime import datetime, date
 from dataclasses import fields, is_dataclass, Field, MISSING, dataclass, asdict
 from uuid import UUID
 from enum import Enum
 import warnings
 
-from typing_extensions import Final, Literal
+try:
+    from typing import Final, Literal  # type: ignore
+except ImportError:
+    from typing_extensions import Final, Literal  # type: ignore
 
 
-from dataclasses_jsonschema.field_types import (  # noqa: F401
-    FieldEncoder, DateTimeFieldEncoder, UuidField, DecimalField,
-    IPv4AddressField, IPv6AddressField, DateTimeField
+from .field_types import (  # noqa: F401
+    FieldEncoder, DateFieldEncoder, DateTimeFieldEncoder, UuidField, DecimalField,
+    IPv4AddressField, IPv6AddressField, DateTimeField, UUID_REGEX
 )
-from dataclasses_jsonschema.type_defs import JsonDict, SchemaType, JsonSchemaMeta, _NULL_TYPE, NULL  # noqa: F401
+from .type_defs import JsonDict, SchemaType, JsonSchemaMeta, _NULL_TYPE, NULL  # noqa: F401
+from .type_hints import get_class_type_hints
 
 try:
     import fastjsonschema
@@ -43,14 +47,22 @@ JSON_ENCODABLE_TYPES = {
 SEQUENCE_TYPES = {
     'Sequence': list,
     'List': list,
-    'Set': set
+    'Set': set,
+    'set': set,
+    'list': list
 }
+MAPPING_TYPES = ('Dict', 'Mapping', 'dict')
+TUPLE_TYPES = ('Tuple', 'tuple')
 
 
 class ValidationError(Exception):
     pass
 
 
+class UnknownEnumValueError(ValueError):
+    pass
+
+
 def is_enum(field_type: Any):
     return issubclass_safe(field_type, Enum)
 
@@ -138,6 +150,12 @@ def _to_camel_case(value: str) -> str:
         return value
 
 
+@dataclass(frozen=True)
+class SchemaOptions:
+    schema_type: SchemaType
+    validate_enums: bool = True
+
+
 @dataclass
 class JsonSchemaField:
     field: Field
@@ -178,6 +196,7 @@ class JsonSchemaMixin:
     convert to and from JSON encodable dicts with validation against the schema
     """
     _field_encoders: Dict[Type, FieldEncoder] = {
+        date: DateFieldEncoder(),
         datetime: DateTimeFieldEncoder(),
         UUID: UuidField(),
         Decimal: DecimalField(),
@@ -186,9 +205,9 @@ class JsonSchemaMixin:
     }
 
     # Cache of the generated schema
-    __schema: Dict[SchemaType, JsonDict]
-    __compiled_schema: Dict[SchemaType, Callable]
-    __definitions: Dict[SchemaType, JsonDict]
+    __schema: Dict[SchemaOptions, JsonDict]
+    __compiled_schema: Dict[SchemaOptions, Callable]
+    __definitions: Dict[SchemaOptions, JsonDict]
     # Cache of field encode / decode functions
     __encode_cache: Dict[Any, _ValueEncoder]
     __decode_cache: Dict[Any, _ValueDecoder]
@@ -225,6 +244,8 @@ class JsonSchemaMixin:
                 klass for klass in cls.__bases__ if is_dataclass(klass) and issubclass(klass, JsonSchemaMixin)
             ]
             if len(dataclass_bases) > 0:
+                if not allow_additional_props:
+                    raise TypeError("Dataclass inheritance and additional_props_false=False not currently supported")
                 base_discriminators = [
                     base._discriminator() for base in dataclass_bases if base._discriminator() is not None
                 ]
@@ -275,7 +296,11 @@ class JsonSchemaMixin:
             elif is_final(field_type):
                 def encoder(ft, val, o): return cls._encode_field(unwrap_final(ft), val, o)
             elif is_enum(field_type):
-                def encoder(_, v, __): return v.value
+                def encoder(_, v, __):
+                    try:
+                        return v.value
+                    except AttributeError as e:
+                        raise UnknownEnumValueError(f'Unknown enum value: {v}') from e
             elif field_type_name == 'Union':
                 # Attempt to encode the field with each union variant.
                 # TODO: Find a more reliable method than this since in the case 'Union[List[str], Dict[str, int]]' this
@@ -285,20 +310,20 @@ class JsonSchemaMixin:
                     try:
                         encoded = cls._encode_field(variant, value, omit_none)
                         break
-                    except (TypeError, AttributeError):
+                    except (TypeError, AttributeError, UnknownEnumValueError):
                         continue
                 if encoded is None:
                     raise TypeError("No variant of '{}' matched the type '{}'".format(field_type, type(value)))
                 return encoded
-            elif field_type_name in ('Mapping', 'Dict'):
+            elif field_type_name in MAPPING_TYPES:
                 def encoder(ft, val, o):
                     return {
                         cls._encode_field(ft.__args__[0], k, o): cls._encode_field(ft.__args__[1], v, o)
                         for k, v in val.items()
                     }
-            elif field_type_name in SEQUENCE_TYPES or (field_type_name == "Tuple" and ... in field_type.__args__):
+            elif field_type_name in SEQUENCE_TYPES or (field_type_name in TUPLE_TYPES and ... in field_type.__args__):
                 def encoder(ft, val, o): return [cls._encode_field(ft.__args__[0], v, o) for v in val]
-            elif field_type_name == 'Tuple':
+            elif field_type_name in TUPLE_TYPES:
                 def encoder(ft, val, o):
                     return [cls._encode_field(ft.__args__[idx], v, o) for idx, v in enumerate(val)]
             elif cls._is_json_schema_subclass(field_type):
@@ -319,15 +344,15 @@ class JsonSchemaMixin:
             dataclass_bases = [
                 klass for klass in cls.__bases__ if is_dataclass(klass) and issubclass(klass, JsonSchemaMixin)
             ]
-            base_field_names = set()
+            base_fields_types = set()
             for base in dataclass_bases:
-                base_field_names |= {f.name for f in fields(base)}
+                base_fields_types |= {(f.name, f.type) for f in fields(base)}
 
             mapped_fields = []
-            type_hints = get_type_hints(cls)
+            type_hints = get_class_type_hints(cls)
             for f in fields(cls):
                 # Skip internal fields
-                if f.name.startswith("__") or (not base_fields and f.name in base_field_names):
+                if f.name.startswith("__") or (not base_fields and (f.name, f.type) in base_fields_types):
                     continue
                 # Note fields() doesn't resolve forward refs
                 f.type = type_hints[f.name]
@@ -355,14 +380,19 @@ class JsonSchemaMixin:
             cls.__mapped_fields = _get_fields_uncached()
         return cls.__mapped_fields  # type: ignore
 
-    def to_dict(self, omit_none: bool = True, validate: bool = False) -> JsonDict:
+    def to_dict(self, omit_none: bool = True, validate: bool = False, validate_enums: bool = True) -> JsonDict:
         """Converts the dataclass instance to a JSON encodable dict, with optional JSON schema validation.
 
         If omit_none (default True) is specified, any items with value None are removed
         """
         data = {}
         for f in self._get_fields():
-            value = self._encode_field(f.field.type, getattr(self, f.field.name), omit_none)
+            value = getattr(self, f.field.name)
+            try:
+                value = self._encode_field(f.field.type, value, omit_none)
+            except UnknownEnumValueError as e:
+                warnings.warn(str(e))
+
             if omit_none and value is None:
                 continue
             if value is NULL:
@@ -373,7 +403,7 @@ class JsonSchemaMixin:
             data[self.__discriminator_name] = self.__class__.__name__
 
         if validate:
-            self._validate(data)
+            self._validate(data, validate_enums)
         return data
 
     @classmethod
@@ -408,27 +438,27 @@ class JsonSchemaMixin:
                         continue
                 if decoded is not None:
                     return decoded
-            elif field_type_name in ('Mapping', 'Dict'):
+            elif field_type_name in MAPPING_TYPES:
                 def decoder(f, ft, val):
                     return {
                         cls._decode_field(f, ft.__args__[0], k): cls._decode_field(f, ft.__args__[1], v)
                         for k, v in val.items()
                     }
-            elif field_type_name in SEQUENCE_TYPES or (field_type_name == "Tuple" and ... in field_type.__args__):
-                seq_type = tuple if field_type_name == "Tuple" else SEQUENCE_TYPES[field_type_name]
+            elif field_type_name in SEQUENCE_TYPES or (field_type_name in TUPLE_TYPES and ... in field_type.__args__):
+                seq_type = tuple if field_type_name in TUPLE_TYPES else SEQUENCE_TYPES[field_type_name]
 
                 def decoder(f, ft, val):
                     return seq_type(cls._decode_field(f, ft.__args__[0], v) for v in val)
-            elif field_type_name == "Tuple":
+            elif field_type_name in TUPLE_TYPES:
                 def decoder(f, ft, val):
                     return tuple(cls._decode_field(f, ft.__args__[idx], v) for idx, v in enumerate(val))
+            elif field_type in cls._field_encoders:
+                def decoder(_, ft, val): return cls._field_encoders[ft].to_python(val)
             elif hasattr(field_type, "__supertype__"):  # NewType field
                 def decoder(f, ft, val):
                     return cls._decode_field(f, ft.__supertype__, val)
             elif is_enum(field_type):
                 def decoder(_, ft, val): return ft(val)
-            elif field_type in cls._field_encoders:
-                def decoder(_, ft, val): return cls._field_encoders[ft].to_python(val)
             elif field_type == Any:
                 def decoder(_, __, val): return val
             if decoder is None:
@@ -438,22 +468,24 @@ class JsonSchemaMixin:
         return decoder(field, field_type, value)
 
     @classmethod
-    def _validate(cls, data: JsonDict):
+    def _validate(cls, data: JsonDict, validate_enums: bool = True):
         try:
             if fast_validation:
                 # TODO: Support validating with other schema types
-                schema_validator = cls.__compiled_schema.get(DEFAULT_SCHEMA_TYPE)
+                schema_validator = cls.__compiled_schema.get(SchemaOptions(DEFAULT_SCHEMA_TYPE, validate_enums))
                 if schema_validator is None:
-                    schema_validator = fastjsonschema.compile(cls.json_schema())
-                    cls.__compiled_schema[DEFAULT_SCHEMA_TYPE] = schema_validator
+                    schema_validator = fastjsonschema.compile(
+                        cls.json_schema(validate_enums=validate_enums), formats={'uuid': UUID_REGEX}
+                    )
+                    cls.__compiled_schema[SchemaOptions(DEFAULT_SCHEMA_TYPE, validate_enums)] = schema_validator
                 schema_validator(data)
             else:
-                validate_func(data, cls.json_schema())
+                validate_func(data, cls.json_schema(validate_enums=validate_enums))
         except JsonSchemaValidationError as e:
             raise ValidationError(str(e)) from e
 
     @classmethod
-    def from_dict(cls: Type[T], data: JsonDict, validate=True) -> T:
+    def from_dict(cls: Type[T], data: JsonDict, validate=True, validate_enums: bool = True) -> T:
         """Returns a dataclass instance with all nested classes converted from the dict given"""
         if cls is JsonSchemaMixin:
             raise NotImplementedError
@@ -467,13 +499,20 @@ class JsonSchemaMixin:
         init_values: Dict[str, Any] = {}
         non_init_values: Dict[str, Any] = {}
         if validate:
-            cls._validate(data)
+            cls._validate(data, validate_enums)
 
         for f in cls._get_fields():
             values = init_values if f.field.init else non_init_values
             if f.mapped_name in data or (
                     f.field.default == MISSING and f.field.default_factory == MISSING):  # type: ignore
-                values[f.field.name] = cls._decode_field(f.field.name, f.field.type, data.get(f.mapped_name))
+                try:
+                    values[f.field.name] = cls._decode_field(f.field.name, f.field.type, data.get(f.mapped_name))
+                except ValueError:
+                    ftype = unwrap_optional(f.field.type) if is_optional(f.field.type) else f.field.type
+                    if is_enum(ftype):
+                        values[f.field.name] = data.get(f.mapped_name)
+                    else:
+                        raise
 
         # Need to ignore the type error here, since mypy doesn't know that subclasses are dataclasses
         instance = cls(**init_values)  # type: ignore
@@ -513,7 +552,7 @@ class JsonSchemaMixin:
                 values[f.field.name] = ft.from_object(from_value, exclude=sub_exclude)
             elif is_enum(ft):
                 values[f.field.name] = ft(from_value)
-            elif field_type_name == "List" and cls._is_json_schema_subclass(ft.__args__[0]):
+            elif field_type_name in ("List", "list") and cls._is_json_schema_subclass(ft.__args__[0]):
                 values[f.field.name] = [
                     ft.__args__[0].from_object(v, exclude=sub_exclude) for v in from_value
                 ]
@@ -558,38 +597,38 @@ class JsonSchemaMixin:
             if schema_type == SchemaType.OPENAPI_3:
                 field_meta.read_only = field.metadata.get("read_only")
                 if field_meta.read_only and default_value is MISSING:
-                    warnings.warn(f"Read-only fields should have a default value")
+                    warnings.warn("Read-only fields should have a default value")
                 field_meta.write_only = field.metadata.get("write_only")
         return field_meta, required
 
     @classmethod
-    def _get_field_schema(cls, field: Union[Field, Type], schema_type: SchemaType) -> Tuple[JsonDict, bool]:
+    def _get_field_schema(cls, field: Union[Field, Type], schema_options: SchemaOptions) -> Tuple[JsonDict, bool]:
         field_schema: JsonDict = {}
         required = True
 
         if isinstance(field, Field):
             field_type = field.type
-            field_meta, required = cls._get_field_meta(field, schema_type)
+            field_meta, required = cls._get_field_meta(field, schema_options.schema_type)
         else:
             field_type = field
-            field_meta = FieldMeta(schema_type=schema_type)
+            field_meta = FieldMeta(schema_type=schema_options.schema_type)
 
         field_type_name = cls._get_field_type_name(field_type)
         if cls._is_json_schema_subclass(field_type):
-            field_schema = schema_reference(schema_type, field_type_name)
+            field_schema = schema_reference(schema_options.schema_type, field_type_name)
         else:
             # If is optional[...]
             if is_optional(field_type):
-                field_schema = cls._get_field_schema(unwrap_optional(field_type), schema_type)[0]
+                field_schema = cls._get_field_schema(unwrap_optional(field_type), schema_options)[0]
                 required = False
             elif is_nullable(field_type):
-                field_schema, required = cls._get_field_schema(unwrap_nullable(field_type), schema_type)
-                if schema_type == SchemaType.OPENAPI_3:
+                field_schema, required = cls._get_field_schema(unwrap_nullable(field_type), schema_options)
+                if schema_options.schema_type == SchemaType.OPENAPI_3:
                     field_schema["nullable"] = True
-                elif schema_type != SchemaType.SWAGGER_V2:
+                elif schema_options.schema_type != SchemaType.SWAGGER_V2:
                     field_schema = {"oneOf": [field_schema, {"type": "null"}]}
             elif is_final(field_type):
-                field_schema, required = cls._get_field_schema(unwrap_final(field_type), schema_type)
+                field_schema, required = cls._get_field_schema(unwrap_final(field_type), schema_options)
             elif is_literal(field_type):
                 field_schema = {
                     'enum': list(field_type.__args__ if sys.version_info[:2] >= (3, 7) else field_type.__values__)
@@ -606,43 +645,46 @@ class JsonSchemaMixin:
                         field_schema.update(JSON_ENCODABLE_TYPES[member_type])
                     else:
                         field_schema.update(cls._field_encoders[member_types.pop()].json_schema)
-                field_schema['enum'] = values
+                if schema_options.validate_enums:
+                    field_schema['enum'] = values
 
-                # If embedding into a swagger spec add the enum name as an extension.
-                # Note: Unlike swagger, JSON schema does not support extensions
-                if schema_type in (SchemaType.SWAGGER_V2, SchemaType.SWAGGER_V3):
-                    field_schema['x-enum-name'] = field_type_name
+                    # If embedding into a swagger spec add the enum name as an extension.
+                    # Note: Unlike swagger, JSON schema does not support extensions
+                    if schema_options.schema_type in (SchemaType.SWAGGER_V2, SchemaType.SWAGGER_V3):
+                        field_schema['x-enum-name'] = field_type_name
+                    if schema_options.schema_type == SchemaType.SWAGGER_V3:
+                        field_schema['x-module-name'] = field_type.__module__
             elif field_type_name == 'Union':
-                if schema_type == SchemaType.SWAGGER_V2:
+                if schema_options.schema_type == SchemaType.SWAGGER_V2:
                     raise TypeError('Type unions unsupported in Swagger 2.0')
                 field_schema = {
-                    'oneOf': [cls._get_field_schema(variant, schema_type)[0] for variant in field_type.__args__]
+                    'oneOf': [cls._get_field_schema(variant, schema_options)[0] for variant in field_type.__args__]
                 }
-            elif field_type_name in ('Dict', 'Mapping'):
+            elif field_type_name in MAPPING_TYPES:
                 field_schema = {'type': 'object'}
                 if field_type.__args__[1] is not Any:
                     field_schema['additionalProperties'] = cls._get_field_schema(
-                        field_type.__args__[1], schema_type
+                        field_type.__args__[1], schema_options
                     )[0]
-            elif field_type_name in SEQUENCE_TYPES or (field_type_name == "Tuple" and ... in field_type.__args__):
+            elif field_type_name in SEQUENCE_TYPES or (field_type_name in TUPLE_TYPES and ... in field_type.__args__):
                 field_schema = {'type': 'array'}
                 if field_type.__args__[0] is not Any:
-                    field_schema['items'] = cls._get_field_schema(field_type.__args__[0], schema_type)[0]
-                if field_type_name == "Set":
+                    field_schema['items'] = cls._get_field_schema(field_type.__args__[0], schema_options)[0]
+                if field_type_name in ("Set", "set"):
                     field_schema['uniqueItems'] = True
-            elif field_type_name == "Tuple":
+            elif field_type_name in TUPLE_TYPES:
                 tuple_len = len(field_type.__args__)
                 # TODO: How do we handle Optional type within lists / tuples
                 field_schema = {
                     'type': 'array', 'minItems': tuple_len, 'maxItems': tuple_len,
-                    'items': [cls._get_field_schema(type_arg, schema_type)[0] for type_arg in field_type.__args__]
+                    'items': [cls._get_field_schema(type_arg, schema_options)[0] for type_arg in field_type.__args__]
                 }
             elif field_type in JSON_ENCODABLE_TYPES:
                 field_schema.update(JSON_ENCODABLE_TYPES[field_type])
             elif field_type in cls._field_encoders:
                 field_schema.update(cls._field_encoders[field_type].json_schema)
             elif hasattr(field_type, '__supertype__'):  # NewType fields
-                field_schema, _ = cls._get_field_schema(field_type.__supertype__, schema_type)
+                field_schema, _ = cls._get_field_schema(field_type.__supertype__, schema_options)
             else:
                 warnings.warn(f"Unable to create schema for '{field_type_name}'")
 
@@ -652,34 +694,49 @@ class JsonSchemaMixin:
 
     @classmethod
     def _get_field_definitions(cls, field_type: Any, definitions: JsonDict,
-                               schema_type: SchemaType):
+                               schema_options: SchemaOptions):
         field_type_name = cls._get_field_type_name(field_type)
-        if is_optional(field_type) or field_type_name in ('Sequence', 'List', 'Tuple'):
-            cls._get_field_definitions(field_type.__args__[0], definitions, schema_type)
-        elif field_type_name in ('Dict', 'Mapping'):
-            cls._get_field_definitions(field_type.__args__[1], definitions, schema_type)
+        if is_optional(field_type):
+            cls._get_field_definitions(unwrap_optional(field_type), definitions, schema_options)
+        elif field_type_name in SEQUENCE_TYPES:
+            cls._get_field_definitions(field_type.__args__[0], definitions, schema_options)
+        elif field_type_name in MAPPING_TYPES:
+            cls._get_field_definitions(field_type.__args__[1], definitions, schema_options)
         elif field_type_name == 'Union':
             for variant in field_type.__args__:
-                cls._get_field_definitions(variant, definitions, schema_type)
+                cls._get_field_definitions(variant, definitions, schema_options)
         elif cls._is_json_schema_subclass(field_type):
             # Prevent recursion from forward refs & circular type dependencies
             if field_type.__name__ not in definitions:
                 definitions[field_type.__name__] = None
-                definitions.update(field_type.json_schema(embeddable=True, schema_type=schema_type))
+                definitions.update(
+                    field_type.json_schema(
+                        embeddable=True,
+                        schema_type=schema_options.schema_type,
+                        validate_enums=schema_options.validate_enums
+                    )
+                )
 
     @classmethod
-    def all_json_schemas(cls: Type[T], schema_type: SchemaType = DEFAULT_SCHEMA_TYPE) -> JsonDict:
+    def all_json_schemas(
+            cls: Type[T], schema_type: SchemaType = DEFAULT_SCHEMA_TYPE, validate_enums: bool = True
+    ) -> JsonDict:
         """Returns JSON schemas for all subclasses"""
         definitions = {}
         for subclass in cls.__subclasses__():
             if is_dataclass(subclass):
-                definitions.update(subclass.json_schema(embeddable=True, schema_type=schema_type))
+                definitions.update(
+                    subclass.json_schema(embeddable=True, schema_type=schema_type, validate_enums=validate_enums)
+                )
             else:
-                definitions.update(subclass.all_json_schemas(schema_type=schema_type))
+                definitions.update(subclass.all_json_schemas(schema_type=schema_type, validate_enums=validate_enums))
         return definitions
 
     @classmethod
-    def json_schema(cls, embeddable: bool = False, schema_type: SchemaType = DEFAULT_SCHEMA_TYPE, **kwargs) -> JsonDict:
+    def json_schema(
+            cls, embeddable: bool = False, schema_type: SchemaType = DEFAULT_SCHEMA_TYPE,
+            validate_enums: bool = True, **kwargs
+    ) -> JsonDict:
         """Returns the JSON schema for the dataclass, along with the schema of any nested dataclasses
         within the 'definitions' field.
 
@@ -692,35 +749,36 @@ class JsonSchemaMixin:
         if 'swagger_version' in kwargs and kwargs['swagger_version'] is not None:
             schema_type = kwargs['swagger_version']
 
+        schema_options = SchemaOptions(schema_type, validate_enums)
+        if schema_options.schema_type in (SchemaType.SWAGGER_V3, SchemaType.SWAGGER_V2) and not embeddable:
+            schema_options = SchemaOptions(SchemaType.DRAFT_06, validate_enums)
+            warnings.warn("'Swagger schema types unsupported when 'embeddable=False', using 'SchemaType.DRAFT_06'")
+
         if cls is JsonSchemaMixin:
             warnings.warn(
                 "Calling 'JsonSchemaMixin.json_schema' is deprecated. Use 'JsonSchemaMixin.all_json_schemas' instead",
                 DeprecationWarning
             )
-            return cls.all_json_schemas(schema_type)
+            return cls.all_json_schemas(schema_options.schema_type, validate_enums)
 
         definitions: JsonDict = {}
-        if schema_type not in cls.__definitions:
-            cls.__definitions[schema_type] = definitions
+        if schema_options not in cls.__definitions:
+            cls.__definitions[schema_options] = definitions
         else:
-            definitions = cls.__definitions[schema_type]
+            definitions = cls.__definitions[schema_options]
 
-        if schema_type in (SchemaType.SWAGGER_V3, SchemaType.SWAGGER_V2) and not embeddable:
-            schema_type = SchemaType.DRAFT_06
-            warnings.warn("'Swagger schema types unsupported when 'embeddable=False', using 'SchemaType.DRAFT_06'")
-
-        if cls.__schema is not None and schema_type in cls.__schema:
-            schema = cls.__schema[schema_type]
+        if schema_options in cls.__schema:
+            schema = cls.__schema[schema_options]
         else:
             properties = {}
             required = []
             for f in cls._get_fields(base_fields=False):
-                properties[f.mapped_name], is_required = cls._get_field_schema(f.field, schema_type)
+                properties[f.mapped_name], is_required = cls._get_field_schema(f.field, schema_options)
                 if f.is_property:
                     properties[f.mapped_name]["readOnly"] = True
-                cls._get_field_definitions(f.field.type, definitions, schema_type)
+                cls._get_field_definitions(f.field.type, definitions, schema_options)
                 # Only add 'readOnly' properties to required for OpenAPI 3
-                if is_required and (not f.is_property or schema_type == SchemaType.OPENAPI_3):
+                if is_required and (not f.is_property or schema_options.schema_type == SchemaType.OPENAPI_3):
                     required.append(f.mapped_name)
             schema = {
                 'type': 'object',
@@ -728,13 +786,17 @@ class JsonSchemaMixin:
                 'properties': properties
             }
 
+            if schema_options.schema_type == SchemaType.OPENAPI_3:
+                schema['x-module-name'] = cls.__module__
+
             if not cls.__allow_additional_props:
                 schema["additionalProperties"] = False
 
             if cls.__discriminator_name is not None and \
-                    schema_type == SchemaType.OPENAPI_3 and \
+                    schema_options.schema_type == SchemaType.OPENAPI_3 and \
                     not cls.__discriminator_inherited:
                 schema['discriminator'] = {'propertyName': cls.__discriminator_name}
+                properties[cls.__discriminator_name] = {"type": "string"}
                 required.append(cls.__discriminator_name)
 
             # Needed for Draft 04 backwards compatibility
@@ -746,21 +808,29 @@ class JsonSchemaMixin:
             ]
             if len(dataclass_bases) > 0:
                 schema = {
-                    "allOf": [schema_reference(schema_type, base.__name__) for base in dataclass_bases] + [schema]
+                    "allOf": [
+                        schema_reference(schema_options.schema_type, base.__name__) for base in dataclass_bases
+                    ] + [schema]
                 }
                 for base in dataclass_bases:
-                    definitions.update(base.json_schema(embeddable=True, schema_type=schema_type))
+                    definitions.update(
+                        base.json_schema(
+                            embeddable=True,
+                            schema_type=schema_options.schema_type,
+                            validate_enums=schema_options.validate_enums
+                        )
+                    )
 
             if cls.__doc__:
                 schema['description'] = cls.__doc__
 
-            cls.__schema[schema_type] = schema
+            cls.__schema[schema_options] = schema
 
         if embeddable:
             return {**definitions, cls.__name__: schema}
         else:
             schema_uri = 'http://json-schema.org/draft-06/schema#'
-            if schema_type == SchemaType.DRAFT_04:
+            if schema_options.schema_type == SchemaType.DRAFT_04:
                 schema_uri = 'http://json-shema.org/draft-04/schema#'
 
             full_schema = {**schema, **{'$schema': schema_uri}}
