@@ -1,19 +1,20 @@
 import datetime
 from _decimal import Decimal
+from copy import deepcopy
 from enum import Enum
 
 from dataclasses import dataclass, field
 from ipaddress import IPv4Address, IPv6Address
-from typing import List, NewType, Optional, Union, Set, Any
+from typing import List, NewType, Optional, Union, Set, Any, cast
 from typing_extensions import Final, Literal
 from uuid import UUID
 
-from dataclasses_jsonschema.type_defs import Nullable, NULL
+from dataclasses_jsonschema.type_defs import Nullable, NULL, JsonDict
 from .conftest import Foo, Point, Recursive, OpaqueData, ShoppingCart, Product, ProductList, SubSchemas, Bar, Weekday, \
     JsonSchemaMixin, Zoo, Baz
 import pytest
 
-from dataclasses_jsonschema import SchemaType, ValidationError, DecimalField, JsonSchemaMeta
+from dataclasses_jsonschema import SchemaType, ValidationError, DecimalField, JsonSchemaMeta, FieldEncoder
 
 FOO_SCHEMA = {
     'description': 'A foo that foos',
@@ -178,7 +179,13 @@ def test_embeddable_json_schema():
     assert expected == Foo.json_schema(embeddable=True)
     expected = {'Point': POINT_SCHEMA, 'Foo': SWAGGER_V2_FOO_SCHEMA}
     assert expected == SubSchemas.all_json_schemas(schema_type=SchemaType.SWAGGER_V2)
-    expected = {'Point': POINT_SCHEMA, 'Foo': SWAGGER_V3_FOO_SCHEMA}
+    expected = {
+        'Point': deepcopy(POINT_SCHEMA),
+        'Foo': deepcopy(SWAGGER_V3_FOO_SCHEMA),
+    }
+    expected['Point']['x-module-name'] = 'tests.conftest'
+    expected['Foo']['x-module-name'] = 'tests.conftest'
+    expected['Foo']['properties']['d']['x-module-name'] = 'tests.conftest'
     assert expected == SubSchemas.all_json_schemas(schema_type=SchemaType.SWAGGER_V3)
     expected = {
         'Point': POINT_SCHEMA,
@@ -441,9 +448,11 @@ def test_field_metadata():
     expected_full_schema = compose_schema(expected_schema)
     assert Test.json_schema() == expected_full_schema
     expected_schema['properties']['name']['x-field-group'] = 1
+    expected_schema['x-module-name'] = 'tests.test_core'
     assert Test.json_schema(schema_type=SchemaType.OPENAPI_3, embeddable=True) == {'Test': expected_schema}
     expected_schema['properties']['name']['example'] = 'foo'
     del expected_schema['properties']['name']['examples']
+    del expected_schema['x-module-name']
     assert Test.json_schema(schema_type=SchemaType.SWAGGER_V2, embeddable=True) == {'Test': expected_schema}
 
 
@@ -632,7 +641,8 @@ def test_nullable_field():
             "name": {"type": "string"},
             "manager": {"type": "string", "nullable": True}
         },
-        "required": ["name"]
+        "required": ["name"],
+        "x-module-name": "tests.test_core"
     }
     expected_json_schema = compose_schema({
         "type": "object",
@@ -696,16 +706,21 @@ def test_discriminators():
                 {
                     "type": "object",
                     "properties": {"breed": {"type": "string"}},
-                    "required": ["breed"]
+                    "required": ["breed"],
+                    "x-module-name": "tests.test_core",
                 },
             ]
         },
         "Pet": {
             "description": "A generic pet",
             "type": "object",
-            "properties": {"name": {"type": "string"}},
+            "properties": {
+                "PetType": {"type": "string"},
+                "name": {"type": "string"}
+            },
             "required": ["name", "PetType"],
-            "discriminator": {"propertyName": "PetType"}
+            "discriminator": {"propertyName": "PetType"},
+            "x-module-name": "tests.test_core",
         }
     }
 
@@ -846,3 +861,120 @@ def test_property_serialisation_all_properties():
         "required": ["width", "height"]
     })
     assert Rectangle.json_schema() == expected_schema
+
+
+def test_inherited_field_narrowing():
+    @dataclass
+    class BaseObject(JsonSchemaMixin):
+        """Base"""
+        other: float
+        field: str
+
+    @dataclass
+    class NarrowedObject(BaseObject, JsonSchemaMixin):
+        """Narrowed"""
+        field: Literal['staticstr']
+
+    expected_schema = compose_schema({
+      "allOf": [
+        {
+          "$ref": "#/definitions/BaseObject"
+        },
+        {
+          "type": "object",
+          "required": ["field"],
+          "properties": {
+            "field": {"enum": ["staticstr"]}
+          }
+        }
+      ],
+      "description": "Narrowed",
+      "$schema": "http://json-schema.org/draft-06/schema#",
+      "definitions": {
+        "BaseObject": {
+          "type": "object",
+          "required": ["other", "field"],
+          "properties": {
+            "field": {"type": "string"},
+            "other": {"type": "number"}
+          },
+          "description": "Base"
+        }
+      }
+    })
+
+    assert NarrowedObject.json_schema() == expected_schema
+
+
+def test_unrecognized_enum_value():
+    class PetType(Enum):
+        CAT = "cat"
+        DOG = "dog"
+
+    class FoodType(Enum):
+        KIBBLE = "kibble"
+        FRESH = "fresh"
+
+    @dataclass
+    class Pet(JsonSchemaMixin):
+        name: str
+        type: PetType
+        favourite_food: Optional[FoodType] = None
+
+    p = Pet.from_dict({'name': 'snakey', 'type': 'python', 'favourite_food': 'mice'}, validate_enums=False)
+    assert p.type == "python"
+    assert p.favourite_food == "mice"
+
+    with pytest.warns(UserWarning):
+        assert p.to_dict() == {'name': 'snakey', 'type': 'python', 'favourite_food': 'mice'}
+
+
+def test_inheritance_and_additional_properties_disallowed():
+    @dataclass
+    class Pet(JsonSchemaMixin):
+        name: str
+
+    # Currently this should raise an error until https://github.com/s-knibbs/dataclasses-jsonschema/issues/111
+    # is implemented
+    with pytest.raises(TypeError):
+        @dataclass
+        class Cat(Pet, allow_additional_props=False):
+            hunting_skill: str
+
+
+def test_newtype_decoding():
+    StrippedString = NewType('StrippedString', str)
+
+    class StrippedStringField(FieldEncoder[StrippedString, str]):
+
+        def to_python(self, value: str) -> StrippedString:
+            return cast(StrippedString, value.strip())
+
+        @property
+        def json_schema(self) -> JsonDict:
+            return {'type': 'string'}
+
+    JsonSchemaMixin.register_field_encoders({StrippedString: StrippedStringField()})
+
+    @dataclass
+    class Pet(JsonSchemaMixin):
+        name: StrippedString
+        type: str
+
+    p = Pet.from_dict({'name': '  Fido ', 'type': 'dog'})
+    assert p.name == 'Fido'
+
+
+def test_module_name_extension():
+    class PetType(Enum):
+        DOG = "dog"
+        CAT = "cat"
+
+    @dataclass
+    class Pet(JsonSchemaMixin):
+        name: str
+        type: PetType
+
+    schema = Pet.json_schema(embeddable=True, schema_type=SchemaType.OPENAPI_3)
+    assert schema['Pet']['x-module-name'] == 'tests.test_core'
+    assert schema['Pet']['properties']['type']['x-module-name'] == 'tests.test_core'
